// Copyright (c) 2003-present, Jodd Team (http://jodd.org)
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

package jodd.vtor;

import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;

public class ValidationConstraintContextTest {

    @Test
    public void testConstructor() throws Exception {
        Vtor vtor = mock(Vtor.class);
        Object target = new Object();
        ValidationConstraintContext context = new ValidationConstraintContext(vtor, target, "niceConstraint");
        assertEquals(context.getName(), "niceConstraint");
        assertEquals(context.getTarget(), target);
        assertEquals(context.getValidator(), vtor);
    }

    @Test
    public void testValidateWithin() throws Exception {
        Vtor vtor = mock(Vtor.class);
        Object target = new Object();
        ValidationConstraintContext context = new ValidationConstraintContext(vtor, target, "niceConstraint");

        ValidationContext vctx = mock(ValidationContext.class);
        Object value = new Object();
        //when
        context.validateWithin(vctx, value);
        //then
        verify(vtor).validate(eq(vctx), eq(value), eq("niceConstraint"));
    }
}